/* This file is generated by glib-genmarshal, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */
#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

#define _gtk_source_marshal_BOOLEAN__BOXED_BOXED	g_cclosure_marshal_BOOLEAN__BOXED_BOXED
#define _gtk_source_marshal_BOOLEAN__BOXED_BOXEDv	g_cclosure_marshal_BOOLEAN__BOXED_BOXEDv

extern
void _gtk_source_marshal_BOOLEAN__BOXED_BOXED_BOXED (GClosure     *closure,
                                                     GValue       *return_value,
                                                     guint         n_param_values,
                                                     const GValue *param_values,
                                                     gpointer      invocation_hint,
                                                     gpointer      marshal_data);
extern
void _gtk_source_marshal_BOOLEAN__BOXED_BOXED_BOXEDv (GClosure *closure,
                                                      GValue   *return_value,
                                                      gpointer  instance,
                                                      va_list   args,
                                                      gpointer  marshal_data,
                                                      int       n_params,
                                                      GType    *param_types);

extern
void _gtk_source_marshal_BOOLEAN__BOXED_BOXED_INT_INT_OBJECT (GClosure     *closure,
                                                              GValue       *return_value,
                                                              guint         n_param_values,
                                                              const GValue *param_values,
                                                              gpointer      invocation_hint,
                                                              gpointer      marshal_data);
extern
void _gtk_source_marshal_BOOLEAN__BOXED_BOXED_INT_INT_OBJECTv (GClosure *closure,
                                                               GValue   *return_value,
                                                               gpointer  instance,
                                                               va_list   args,
                                                               gpointer  marshal_data,
                                                               int       n_params,
                                                               GType    *param_types);

extern
void _gtk_source_marshal_STRING__OBJECT (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
                                         const GValue *param_values,
                                         gpointer      invocation_hint,
                                         gpointer      marshal_data);
extern
void _gtk_source_marshal_STRING__OBJECTv (GClosure *closure,
                                          GValue   *return_value,
                                          gpointer  instance,
                                          va_list   args,
                                          gpointer  marshal_data,
                                          int       n_params,
                                          GType    *param_types);

extern
void _gtk_source_marshal_VOID__BOXED_BOXED (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
                                            const GValue *param_values,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
extern
void _gtk_source_marshal_VOID__BOXED_BOXEDv (GClosure *closure,
                                             GValue   *return_value,
                                             gpointer  instance,
                                             va_list   args,
                                             gpointer  marshal_data,
                                             int       n_params,
                                             GType    *param_types);

extern
void _gtk_source_marshal_VOID__BOXED_BOXED_BOXED (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
                                                  const GValue *param_values,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
extern
void _gtk_source_marshal_VOID__BOXED_BOXED_BOXEDv (GClosure *closure,
                                                   GValue   *return_value,
                                                   gpointer  instance,
                                                   va_list   args,
                                                   gpointer  marshal_data,
                                                   int       n_params,
                                                   GType    *param_types);

extern
void _gtk_source_marshal_VOID__BOXED_BOXED_FLAGS (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
                                                  const GValue *param_values,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
extern
void _gtk_source_marshal_VOID__BOXED_BOXED_FLAGSv (GClosure *closure,
                                                   GValue   *return_value,
                                                   gpointer  instance,
                                                   va_list   args,
                                                   gpointer  marshal_data,
                                                   int       n_params,
                                                   GType    *param_types);

extern
void _gtk_source_marshal_VOID__BOXED_UINT_FLAGS_INT (GClosure     *closure,
                                                     GValue       *return_value,
                                                     guint         n_param_values,
                                                     const GValue *param_values,
                                                     gpointer      invocation_hint,
                                                     gpointer      marshal_data);
extern
void _gtk_source_marshal_VOID__BOXED_UINT_FLAGS_INTv (GClosure *closure,
                                                      GValue   *return_value,
                                                      gpointer  instance,
                                                      va_list   args,
                                                      gpointer  marshal_data,
                                                      int       n_params,
                                                      GType    *param_types);

extern
void _gtk_source_marshal_VOID__BOXED_BOXED_UINT_FLAGS_INT (GClosure     *closure,
                                                           GValue       *return_value,
                                                           guint         n_param_values,
                                                           const GValue *param_values,
                                                           gpointer      invocation_hint,
                                                           gpointer      marshal_data);
extern
void _gtk_source_marshal_VOID__BOXED_BOXED_UINT_FLAGS_INTv (GClosure *closure,
                                                            GValue   *return_value,
                                                            gpointer  instance,
                                                            va_list   args,
                                                            gpointer  marshal_data,
                                                            int       n_params,
                                                            GType    *param_types);

extern
void _gtk_source_marshal_VOID__BOXED_ENUM (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
                                           const GValue *param_values,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
extern
void _gtk_source_marshal_VOID__BOXED_ENUMv (GClosure *closure,
                                            GValue   *return_value,
                                            gpointer  instance,
                                            va_list   args,
                                            gpointer  marshal_data,
                                            int       n_params,
                                            GType    *param_types);

extern
void _gtk_source_marshal_VOID__BOXED_INT (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
                                          const GValue *param_values,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
extern
void _gtk_source_marshal_VOID__BOXED_INTv (GClosure *closure,
                                           GValue   *return_value,
                                           gpointer  instance,
                                           va_list   args,
                                           gpointer  marshal_data,
                                           int       n_params,
                                           GType    *param_types);

extern
void _gtk_source_marshal_VOID__ENUM_INT (GClosure     *closure,
                                         GValue       *return_value,
                                         guint         n_param_values,
                                         const GValue *param_values,
                                         gpointer      invocation_hint,
                                         gpointer      marshal_data);
extern
void _gtk_source_marshal_VOID__ENUM_INTv (GClosure *closure,
                                          GValue   *return_value,
                                          gpointer  instance,
                                          va_list   args,
                                          gpointer  marshal_data,
                                          int       n_params,
                                          GType    *param_types);

extern
void _gtk_source_marshal_VOID__OBJECT_BOXED (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
                                             const GValue *param_values,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
extern
void _gtk_source_marshal_VOID__OBJECT_BOXEDv (GClosure *closure,
                                              GValue   *return_value,
                                              gpointer  instance,
                                              va_list   args,
                                              gpointer  marshal_data,
                                              int       n_params,
                                              GType    *param_types);

extern
void _gtk_source_marshal_VOID__OBJECT_UINT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
                                            const GValue *param_values,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
extern
void _gtk_source_marshal_VOID__OBJECT_UINTv (GClosure *closure,
                                             GValue   *return_value,
                                             gpointer  instance,
                                             va_list   args,
                                             gpointer  marshal_data,
                                             int       n_params,
                                             GType    *param_types);


G_END_DECLS
